@@ -40,8 +40,6 @@
 #include "discid/discid_private.h"
 #include "unix.h"
 
-#define MB_DEFAULT_DEVICE	"/dev/cdrom"
-
 
 /* timeout better shouldn't happen for scsi commands -> device is reset */
 #define DEFAULT_TIMEOUT 30000	/* in ms */
@@ -50,6 +48,67 @@
 #define SG_MAX_SENSE 16
 #endif
 
+#define MB_DEFAULT_DEVICE "/dev/cdrom"
+#define MAX_DEV_LEN 50
+
+#if (defined(__GNUC__) && (__GNUC__ >= 4)) || defined(__clang__)
+#	define THREAD_LOCAL __thread
+#else
+#	define THREAD_LOCAL
+#endif
+
+static THREAD_LOCAL char default_device[MAX_DEV_LEN] = "";
+
+
+static int get_device(int number, char *device, int device_len) {
+	FILE *proc_file;
+	char *current_device;
+	char *lineptr = NULL;
+	char *saveptr = NULL;
+	size_t bufflen;
+	int i, count, counter;
+	int return_value = 0;
+
+	proc_file = fopen("/proc/sys/dev/cdrom/info", "r");
+	if (proc_file != NULL) {
+		/* skip to line containing device names */
+		do {
+			if (getline(&lineptr, &bufflen, proc_file) < 0) {
+				return 0;
+			}
+		} while (strstr(lineptr, "drive name:") == NULL);
+
+		/* count number of devices = number of tabs - 1*/
+		count = -1;
+		for (i = 0; i < strlen(lineptr); i++) {
+			if (lineptr[i] == '\t') count++;
+		}
+
+		/* go through devices, they are in reverse order */
+		current_device = strtok_r(lineptr, "\t", &saveptr);
+		/* skip column title */
+		current_device = strtok_r(NULL, "\t", &saveptr);
+		counter = count;
+		while (current_device != NULL && counter >= number) {
+			if (counter == number) {
+				snprintf(device, device_len,
+					 "/dev/%s", current_device);
+				return_value = 1;
+			}
+			/* go to next in list */
+			current_device = strtok_r(NULL, "\t", &saveptr);
+			counter--;
+		}
+
+		/* trim the trailing \n for the last entry = first device */
+		if (default_device[strlen(default_device)-1] == '\n') {
+			default_device[strlen(default_device)-1] = '\0';
+		}
+		free(lineptr);
+		fclose(proc_file);
+	}
+	return return_value;
+}
 
 int mb_disc_unix_read_toc_header(int fd, mb_disc_toc *toc) {
 	struct cdrom_tochdr th;
@@ -86,12 +145,22 @@ int mb_disc_unix_read_toc_entry(int fd, int track_num, mb_disc_toc_track *track)
 }
 
 char *mb_disc_get_default_device_unportable(void) {
-	return MB_DEFAULT_DEVICE;
+	/* prefer the default device symlink to the internal names */
+	if (mb_disc_unix_exists(MB_DEFAULT_DEVICE)) {
+		return MB_DEFAULT_DEVICE;
+	} else {
+		if (get_device(1, default_device, MAX_DEV_LEN)) {
+			return default_device;
+		} else {
+			return MB_DEFAULT_DEVICE;
+		}
+	}
 }
 
 void mb_disc_unix_read_mcn(int fd, mb_disc_private *disc)
 {
 	struct cdrom_mcn mcn;
+	memset(&mcn, 0, sizeof mcn);
 
 	if(ioctl(fd, CDROM_GET_MCN, &mcn) == -1) {
 		fprintf(stderr, "Warning: Unable to read the disc's media catalog number.\n");
@@ -184,4 +253,24 @@ int mb_disc_has_feature_unportable(enum discid_feature feature) {
 	}
 }
 
+int mb_disc_read_unportable(mb_disc_private *disc, const char *device,
+			    unsigned int features) {
+	char device_name[MAX_DEV_LEN] = "";
+	int device_number;
+
+	device_number = (int) strtol(device, NULL, 10);
+	if (device_number > 0) {
+		if(!get_device(device_number, device_name, MAX_DEV_LEN)) {
+			snprintf(disc->error_msg, MB_ERROR_MSG_LENGTH,
+				 "cannot find cd device with the number '%d'",
+				 device_number);
+			return 0;
+		} else {
+			return mb_disc_unix_read(disc, device_name, features);
+		}
+	} else {
+		return mb_disc_unix_read(disc, device, features);
+	}
+}
+
 /* EOF */
